local _, BohemkaDKP = ...
local LibDeflate = LibStub:GetLibrary("LibDeflate")
local LibSerialize = LibStub("LibSerialize")

function BohemkaDKP:EncodePayload(data)
    local payload = table.concat(data, "\n")
    local serialized = LibSerialize:Serialize(payload)
    local compressed = LibDeflate:CompressDeflate(serialized)
    return LibDeflate:EncodeForWoWAddonChannel(compressed)
end
function BohemkaDKP:DecodePayload(data)
    data = LibDeflate:DecodeForWoWAddonChannel(data)
    data = LibDeflate:DecompressDeflate(data)
    local _, deserialized = LibSerialize:Deserialize(data)
    return {strsplit("\n", deserialized)}
end
function BohemkaDKP:PreparePayloadForSend(data)
    local id = math.random(100)
    data = self:EncodePayload(data)
    local chunks = self:splitStringByChunks(data, 240 - CHUNK_HEAD_SIZE)
    local totalSize = #data + (#chunks * (self.cpsOverhead + CHUNK_HEAD_SIZE))
    self:Debug("Sending data with size of", totalSize, "|| chunks:", #chunks)
    self:Debug("Sending will take", totalSize / self.cpsLimit, "seconds")
    self.chunksToSend[id] = chunks
    return id, chunks
end
function BohemkaDKP:SendPayloadTo(target, data)
    id, data = self:PreparePayloadForSend(data)
    self:SendEventTo(target, self.EVENT.PAYLOAD_START, #data, id)
end
function BohemkaDKP:SendPayload(channel, data)
    id, data = self:PreparePayloadForSend(data)
    self:SendEvent(channel, self.EVENT.PAYLOAD_START, #data, id)
end
function BohemkaDKP:BroadcastPayload(channel, data)
    id, data = self:PreparePayloadForSend(data)
    self:SendEvent(channel, self.EVENT.BROADCAST_START, #data, id)
    C_Timer.After(3, function ()
        BohemkaDKP:StartBroadcastPayload(id)
    end)
end
