---
--- Generated by EmmyLua(https://github.com/EmmyLua)
--- Created by brode.
--- DateTime: 15.07.2021 17:54
---
local _, BohemkaDKP = ...

function BohemkaDKP:ShareProfessions()
    local skills = {}
    local skillRanks = {}
    for skillIndex = 1, GetNumSkillLines() do
        local skillName, isHeader, _, skillRank, _, _,
        skillMaxRank, _, _, _, _, _,
        _ = GetSkillLineInfo(skillIndex)
        if not isHeader then
            if BohemkaDKP.PROFESSIONS[skillName] then
                skillRanks[skillName] = skillRank.."-"..skillMaxRank
            end
        end
    end
    local _, _, offset, numSlots = GetSpellTabInfo(1)
    local foundMining = false
    for j = offset+1, offset+numSlots do
        local name, _, _, _, _, _, spellId = GetSpellInfo(GetSpellBookItemName(j, BOOKTYPE_SPELL))
        if BohemkaDKP.PROFESSIONS[name] then
            local data = spellId
            if name ~= "Mining" then
                if name == "Find Herbs" then
                    data = 9134
                    name = "Herbalism"
                elseif name == "Find Minerals" then
                    data = 29354
                    name = "Mining"
                    foundMining = true
                end
                if skillRanks[name] then
                    data = data.."-"..skillRanks[name]
                end
                table.insert(skills, data)
            end
        end
    end
    local payload = table.concat(skills, ",")
    self:SendEvent("GUILD", self.EVENT.PROFESSION_INFO, payload)
end

function BohemkaDKP:OnProfessionInfo(payload, sender)
    local spellIds = { strsplit(",", payload) }
    if sender then
        local tmp = {}
        for _, v in ipairs(spellIds) do
            local spellId, spellRank, spellMaxRank = strsplit("-", v)
            spellId = tonumber(spellId)
            spellRank = tonumber(spellRank)
            spellMaxRank = tonumber(spellMaxRank)
            local name, _, icon = GetSpellInfo(spellId)
            local order = BohemkaDKP.PROFESSIONS[name]
            if order then
                table.insert(tmp, {name = name, order = order, id = spellId, icon = icon, rank = spellRank, maxRank = spellMaxRank})
            end
        end
        table.sort(tmp, function (a, b) return a.order < b.order end)
        RosterData.professions[sender] = tmp
    end
end

function BohemkaDKP:GetPlayerProfessions(fullName)
    return RosterData.professions[fullName] or {}
end
function BohemkaDKP:RequestProfessionInfo()
    self:SendEvent("GUILD", self.EVENT.PROFESSION_INFO_REQUEST)
end

function BohemkaDKP:OnProfessionInfoRequest(sender)
    if sender == self:GetPlayerName(true) then
        return
    end
    self:ShareProfessions()

end

function BohemkaDKP:OnCraft(profName, craftName, craftType, numAvailable, icon, desc, cooldown, reagents, link, id, minMade, maxMade, sender)
    desc = self:decodeBase64(desc)
    local reagents_t = {}
    if not reagents or not sender then
        return
    end
    reagents = {strsplit("*", reagents)}
    for _, reagent in pairs(reagents) do
        local reagentTexture, reagentCount, playerReagentCount, reagentLink = strsplit("~", reagent)
        table.insert(reagents_t, {
            texture = tonumber(reagentTexture),
            count = tonumber(reagentCount),
            playerCount = tonumber(playerReagentCount),
            link = reagentLink
        })
    end

    if not Crafts[sender] then
        Crafts[sender] = {}
    end
    if not Crafts[sender][profName] then
        Crafts[sender][profName] = {}
    end
    Crafts[sender][profName][craftName] = {
        available = tonumber(numAvailable),
        icon = icon,
        desc = desc,
        type = craftType,
        cooldown = tonumber(cooldown),
        reagents = reagents_t,
        link = link,
        id = tonumber(id),
        min = tonumber(minMade),
        max = tonumber(maxMade),
    }
end

local craftFrame = CreateFrame("Frame")
local craftFrame2 = CreateFrame("Frame")

function BohemkaDKP:ShareCrafts()
    local numCrafts = GetNumCrafts()
    local payload = {}
    local profName, _, _ = GetCraftDisplaySkillLine()
    if not profName then
        return
    end
    self.sharedCrafts[profName] = GetServerTime()
    local i = 1
    local wait = 0
    craftFrame2:SetScript("OnUpdate", function(_, elapsed)
        if wait > 0 then
            wait = wait - elapsed
            return
        end
        if i > numCrafts then
            craftFrame2:SetScript("OnUpdate", nil)
            BohemkaDKP:BroadcastPayload("GUILD", payload)
            return
        end
        if not BohemkaDKP:ValidateCraft(profName) then
            return
        end
        local craftName, _, craftType, numAvailable = GetCraftInfo(i)
        local numReagents = GetCraftNumReagents(i)
        local minMade,maxMade = GetCraftNumMade(i)
        local icon = GetCraftIcon(i)
        local desc = GetCraftDescription(i) or ""
        desc = self:encodeBase64(desc)
        local link = GetCraftItemLink(i)
        local cooldown = GetCraftCooldown(i) or 0
        local reagents = {}
        for j=1, numReagents do
            local _, reagentTexture, reagentCount, playerReagentCount = GetCraftReagentInfo(i, j);
            local reagentLink = GetCraftReagentItemLink(i, j)
            table.insert(reagents, table.concat(table.removeNil({reagentTexture, reagentCount, playerReagentCount, reagentLink}), "~"))
        end
        reagents = table.concat(reagents, "*")
        table.insert(payload, self:PreparePayload(self.EVENT.CRAFT, profName, craftName, craftType, numAvailable, icon, desc, cooldown, reagents, link, i, minMade, maxMade))
        i = i + 1
    end)
end

function BohemkaDKP:ShareTradeSkills()
    local numCrafts = GetNumTradeSkills()
    local payload = {}
    local profName = GetTradeSkillLine()
    if not profName then
        return
    end
    self.sharedCrafts[profName] = GetServerTime()
    local i = 1
    local wait = 0
    craftFrame:SetScript("OnUpdate", function(_, elapsed)
        if wait > 0 then
            wait = wait - elapsed
            return
        end
        if i > numCrafts then
            craftFrame:SetScript("OnUpdate", nil)
            BohemkaDKP:BroadcastPayload("GUILD", payload)
            return
        end
        if not BohemkaDKP:ValidateProf(profName) then
            return
        end
        local craftName, craftType, numAvailable = GetTradeSkillInfo(i)
        if craftType ~= "header" then
            local numReagents = GetTradeSkillNumReagents(i)
            local minMade,maxMade = GetTradeSkillNumMade(i)
            local icon = GetTradeSkillIcon(i)
            local desc = " "
            desc = self:encodeBase64(desc)
            local link = GetTradeSkillRecipeLink(i)
            local cooldown = GetTradeSkillCooldown(i) or 0
            local reagents = {}
            for j=1, numReagents do
                local reagentName, reagentTexture, reagentCount, playerReagentCount = GetTradeSkillReagentInfo(i, j);
                local reagentLink = GetTradeSkillReagentItemLink(i, j)
                if reagentName == nil or reagentLink == nil then
                    wait = 0.1
                    return
                end

                table.insert(reagents, table.concat(table.removeNil({reagentTexture, reagentCount, playerReagentCount, reagentLink}), "~"))
            end
            reagents = table.concat(reagents, "*")
            if not BohemkaDKP:ValidateProf(profName) then
                return
            end
            table.insert(payload, self:PreparePayload(self.EVENT.CRAFT, profName, craftName, craftType, numAvailable, icon, desc, cooldown, reagents, link, i, minMade, maxMade))
        end
        i = i + 1
    end)
end
function BohemkaDKP:ValidateProf(profName)
    local profNameTest = GetTradeSkillLine()
    if profNameTest ~= profName then
        BohemkaDKP.sharedCrafts[profName] = nil
        craftFrame:SetScript("OnUpdate", nil)
        return false
    end
    return true
end
function BohemkaDKP:ValidateCraft(profName)
    local profNameTest = GetCraftDisplaySkillLine()
    if profNameTest ~= profName then
        BohemkaDKP.sharedCrafts[profName] = nil
        craftFrame2:SetScript("OnUpdate", nil)
        return false
    end
    return true
end
function BohemkaDKP:ShareCraftsDelayed()
    local profName = GetCraftName()
    if self.sharedCrafts[profName] and GetServerTime() - self.sharedCrafts[profName] < 60 then
        return
    end
    self:ShareCrafts()
end

function BohemkaDKP:ShareTradeSkillsDelayed()
    local profName = GetTradeSkillLine()
    if self.sharedCrafts[profName] and GetServerTime() - self.sharedCrafts[profName] < 60 then
        return
    end
    self:ShareTradeSkills()
end
function BohemkaDKP:ShareCraftHistory()
    local crafts = self:GetPlayerCraftHistory()
    if crafts then
        local payload = {}
        for profName, item in ipairs(crafts) do
            for craftName, data in ipairs(item) do
                local reagents = {}
                for _, reagent in pairs(data.reagents) do
                    table.insert(reagents, {reagent.texture, reagent.count, reagent.playerCount, reagent.link}, "~")
                end
                reagents = table.concat(reagents, "*")
                table.insert(payload, self:PreparePayload(self.EVENT.CRAFT, profName, craftName, data.type, data.available, data.icon, data.desc, data.cooldown, reagents, data.link, data.id, data.min, data.max))
            end
        end
        self:Debug("Sharing craft history")
        BohemkaDKP:BroadcastPayload("GUILD", payload)
    end
end
function BohemkaDKP:GetPlayerCraftHistory(name)
    return Crafts[name or self:GetPlayerName(true)]
end
function BohemkaDKP:FilterCrafts(searchValue, profName, playerName)
    local crafts = self:GetPlayerCraftHistory(playerName)
    local result = {}
    if crafts and crafts[profName] then
        for craftName, data in pairs(crafts[profName]) do
            if string.find(strlower(craftName), searchValue) then
                result[craftName] = data
            end
        end
    end
    return result
end
function BohemkaDKP:FilterCraftPlayers(searchValue)
    local result = {}
    for playerName, professions in pairs(Crafts) do
        for profName, profession in pairs(professions) do
            for craftName, data in pairs(profession) do
                if string.find(strlower(craftName), searchValue) then
                    if not result[playerName] then
                        result[playerName] = {}
                    end
                    if not result[playerName][profName] then
                        result[playerName][profName] = {}
                    end
                    result[playerName][profName][craftName] = data
                end
            end
        end
    end
    return result
end
function BohemkaDKP:GetProfessionIcon(profession)
    return BohemkaDKP.PROFESSION_ICON_OVERRIDE[profession.name] or profession.icon
end
